<?php
/**
 * @file
 * Enables modules and site configuration for a Microserve site installation.
 */

/**
 * Helper function to get the site name from the url.
 *
 * @return string
 *   The current hostname minus the TLD and www. prefix, e.g. 'example' for a
 *   hostname of 'www.example.com'.
 */
function _microserve_profile_site_name($site_name = NULL) {
  if (!isset($site_name)) {
    $site_name = $_SERVER['SERVER_NAME'];
  }

  // Strip www. prefix;
  if (strpos($site_name, 'www.') === 0) {
    $site_name = substr($site_name, 4);
  }

  list($site_name) = explode('.', $site_name);

  return $site_name;
}

/**
 * Implements hook_form_FORM_ID_alter() for install_settings_form().
 *
 * Allows the profile to alter the site settings form. Pretend to be system
 * module to get this to work as it is too early in bootstrap for the standard
 * hook_form_*_alter() functions to work.
 *  *
 * @see https://drupal.org/node/1153646
 */
function system_form_install_settings_form_alter(&$form, &$form_state) {
  $show_message = FALSE;
  foreach (array('mysql', 'pgsql') as $key) {
    if (isset($form['settings'][$key]['username'])) {
      // Generate a database name and user account (to be created).
      $form['settings'][$key]['database']['#default_value'] = _microserve_profile_site_name();
      $form['settings'][$key]['username']['#default_value'] = _microserve_profile_site_name();
      // Generate a random 16 character password to be used for the database user account.
      $form['settings'][$key]['password']['#type'] = 'textfield';
      $form['settings'][$key]['password']['#default_value'] = user_password(16);

      $show_message = TRUE;
    }
  }

  if ($show_message) {
    // Show a help message about setting up a new database for the site.
    $form['settings']['message'] = array(
      '#weight' => -100,
      '#type' => 'markup',
      '#markup' => '<div class="messages status">' . st('Please set up a new database using the following details:') . '</div>',
    );
  }
}

/**
 * Implements hook_form_FORM_ID_alter() for install_configure_form().
 *
 * Allows the profile to alter the site configuration form.
 */
function microserve_form_install_configure_form_alter(&$form, $form_state) {
  // Pre-populate the site name with the server name.
  $form['site_information']['site_name']['#default_value'] = _microserve_profile_site_name();
  $form['admin_account']['account']['name']['#default_value'] = 'admin';

  // Fill in the country and timezone settings for UK.
  $form['server_settings']['site_default_country']['#type'] = 'value';
  $form['server_settings']['site_default_country']['#value'] = 'GB';

  $form['server_settings']['date_default_timezone']['#type'] = 'value';
  $form['server_settings']['date_default_timezone']['#value'] = 'Europe/London';

  unset($form['server_settings']['#type']);

  // Turn off update module.
  $form['update_notifications']['update_status_module']['#type'] = 'value';
  $form['update_notifications']['update_status_module']['#value'] = array(1 => FALSE, 2 => FALSE);
  unset($form['update_notifications']['#type']);
}

/**
 * Implements hook_install_tasks_alter().
 */
function microserve_install_tasks_alter(&$tasks, $install_state) {
  // Replace the "Choose language" installation task provided by Drupal core.
  $tasks['install_select_locale']['function'] = 'microserve_locale_selection';
  $tasks['install_finished']['function'] = 'microserve_install_finished';
}

function microserve_locale_selection(&$install_state) {
  // Set the locale to English.
  $install_state['parameters']['locale'] = 'en';
}

function microserve_install_finished(&$install_state) {
  $message = st('Congratulations, you installed @drupal!', array('@drupal' => drupal_install_profile_distribution_name()));

  // Flush all caches to ensure that any full bootstraps during the installer
  // do not leave stale cached data, and that any content types or other items
  // registered by the installation profile are registered correctly.
  drupal_flush_all_caches();

  // Remember the profile which was used.
  variable_set('install_profile', drupal_get_profile());

  // Installation profiles are always loaded last
  db_update('system')
  ->fields(array('weight' => 1000))
  ->condition('type', 'module')
  ->condition('name', drupal_get_profile())
  ->execute();

  // Cache a fully-built schema.
  drupal_get_schema(NULL, TRUE);

  // Ensure system file directories are setup correctly.
  $public = variable_get('file_public_path', conf_path() . '/files');
  system_check_directory(array('#value' => $public, '#name' => 'file_public_path'));
  system_check_directory(array('#value' => variable_get('file_private_path', $public . '/private'), '#name' => 'file_private_path'));
  system_check_directory(array('#value' => variable_get('file_temporary_path', $public . '/tmp'), '#name' => 'file_temporary_path'));

  if (module_exists('features')) {
    if (!module_exists('microserve_user_roles')) {
      // The user roles feature includes permissions for modules which would be
      // enabled after it in the installation process, so we manually turn it on
      // here instead.
      module_enable(array('microserve_user_roles'));
    }

    // Revert the text format feature. This resets the CKEditor profiles.
    features_revert_module('microserve_text_formats');
  }

  // Add our performance configuration to settings.php.
  $settings = file_get_contents(__DIR__ . '/microserve.settings.php');
  $tokens = array(
    '%profile-name' => _microserve_profile_site_name(),
  );
  $settings = str_replace(array_keys($tokens), array_values($tokens), $settings);
  // Save the changes to settings.php.
  _microserve_file_str_append(conf_path(FALSE) . '/settings.php', $settings);

  // Turn on performance settings.
  variable_set('cache', TRUE);
  variable_set('page_cache_maximum_age', 86400);
  variable_set('preprocess_css', TRUE);
  variable_set('preprocess_js', TRUE);

  // Run cron to populate update status tables (if available) so that users
  // will be warned if they've installed an out of date Drupal version.
  // Will also trigger indexing of profile-supplied content or feeds.
  drupal_cron_run();

  // Set a congratulation message and send the user on to their new site.
  drupal_set_message($message);
  $url = '<front>';
  if (module_exists('styleguide')) {
    $url = 'admin/appearance/styleguide';
  }
  drupal_goto($url);
}
